package
{
	import flash.display.*;
	import flash.events.*;
	import flash.filters.GlowFilter;
	
	import org.papervision3d.cameras.*;
	import org.papervision3d.view.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.objects.primitives.*
	import org.papervision3d.materials.utils.*;
	
	import fl.motion.easing.*
	import com.flashdynamix.motion.Tweensy;
	
	/**
	* Photo Print Effect
	*/
	public class Main extends BasicView 
	{	
		// Particle Separate Number
		static private const MAX_CELL_W:uint = 20;
		static private const MAX_CELL_H:uint = 15;
		
		// Blur Effect Width/height
		static private const EFFECT_W:int = 250;
		static private const EFFECT_H:int = 250;

		// Particle Array
		private var particlePlaneArr:Array /* type of Array */ = [];
		
		// Particle Container
		private var wrap:DisplayObject3D;
		
		// Blur Effect
		private var radiationBlurPlane:Plane;
		private var imgSprite:Sprite = new BitmapImage();
		private var radiationBlurMaterial:BitmapMaterial;
		private var linePlane:Plane;
		private var isMotion:Boolean = false;
		
		/**
		 * Constructor
		 */
		public function Main()
		{
			// init BasicView
			super(720, 480, false, false, CameraType.TARGET);
			
			// init swf
			stage.quality = StageQuality.MEDIUM;
			
			init();
			
			// setting Button Event
			btn.addEventListener(MouseEvent.CLICK, function(event:Event):void
			{
				if (isMotion) return;
				startMotion();
			});
			
			startMotion();
		}
		
		/**
		 * initialize
		 */
		private function init():void
		{
			//--------------------------------
			// Camera
			//--------------------------------
			camera.focus = 200;
			camera.zoom = 1.5;
			
			//--------------------------------
			// Create Photo Particle
			//--------------------------------
			var folder:Sprite = new Sprite();
			imgSprite = new BitmapImage();
			folder.addChild(imgSprite);
			
			// Particle Array
			var bmpDataArr:Array /* type of BitmapData */ = [];
			
			var i:int = 0;
			var j:int = 0;
			
			// save particle
			for (i = 0; i < MAX_CELL_W; i++ )
			{
				for (j = 0; j < MAX_CELL_H; j++ )
				{
					imgSprite.x = - imgSprite.width / MAX_CELL_W * i;
					imgSprite.y = - imgSprite.height / MAX_CELL_H * j;
					
					var bmpData:BitmapData = new BitmapData(
						imgSprite.width / MAX_CELL_W, 
						imgSprite.height/ MAX_CELL_H, 
						false, 
						0xFFFF0000
					);
					
					bmpData.draw(folder);
					bmpDataArr.push(bmpData);
				}
			}
			
			// cratet wrapper object of particle
			wrap = new DisplayObject3D();
			scene.addChild(wrap);
			wrap.visible = false;
			
			// setting position 
			wrap.x = - imgSprite.width / 2;
			wrap.y = - imgSprite.height / 2;
			
			// create 3d plane from photo particle
			for (i = 0; i < MAX_CELL_W; i++ )
			{
				particlePlaneArr[i] /* type of Plane */ = [];
				
				for (j = MAX_CELL_H; j > 0 ; j-- )
				{
					var material:BitmapMaterial = new BitmapMaterial(BitmapData(bmpDataArr.shift()), false);
					
					var plane:Plane = new Plane(
						material, 
						imgSprite.width / MAX_CELL_W, 
						imgSprite.height / MAX_CELL_H, 
						1, 1
					);
					
					plane.x = imgSprite.width / MAX_CELL_W * i + (imgSprite.width / MAX_CELL_W / 2);
					plane.y = imgSprite.height / MAX_CELL_H * j - (imgSprite.height / MAX_CELL_H / 2);
					
					wrap.addChild(plane);
					particlePlaneArr[i][j] = plane;
				}
			}
			
			
			//--------------------------------
			// Create Radiation Blur
			//--------------------------------
			var radiationMaterial:MovieAssetMaterial = new MovieAssetMaterial("RadiationBlur", false, true, false);
			radiationMaterial.opposite = true;
			radiationBlurPlane = new Plane(radiationMaterial, imgSprite.height, imgSprite.height * 2, 6, 6);
			
			// for effect
			radiationBlurPlane.useOwnContainer = true;
			radiationBlurPlane.blendMode = BlendMode.ADD;
			
			// for centering
			radiationBlurPlane.z = - imgSprite.height;
			radiationBlurPlane.localRotationY = 90;
			radiationBlurPlane.localRotationZ = 90;
			
			// add 3D scene
			scene.addChild(radiationBlurPlane);
			radiationBlurPlane.visible = false;
			
			//--------------------------------
			// Create Radiation Blur Start Effect
			//--------------------------------
			var lineMaterial:MovieAssetMaterial = new MovieAssetMaterial("LinePlane", false, false, false, false);
			linePlane = new Plane(lineMaterial, 3, imgSprite.height, 6, 6);
			linePlane.useOwnContainer = true;
			linePlane.filters = [new GlowFilter(0xFFFFFF, 1, 16, 4, 6, 3)];
			linePlane.blendMode = BlendMode.ADD;
			scene.addChild(linePlane)
			linePlane.visible = false;
		}
		
		
			
		/**
		* Start Motion Method
		*/
		private function startMotion():void
		{
			// start PV3D Rendering
			startRendering();
			isMotion = true;
			
			// motion init
			wrap.visible = true;
			radiationBlurPlane.visible = true;
			linePlane.visible = true;
			
			var i:int = 0;
			var j:int = 0;
			var ease:Number = 0.25;
			
			// Particle Photo Effect
			for (i = 0; i < particlePlaneArr.length; i++ )
			{
				for (j = 0; j < particlePlaneArr[i].length; j++ )
				{
					if(particlePlaneArr[i][j] == null ) continue;
					particlePlaneArr[i][j].z = -1250;
					Tweensy.to(
						particlePlaneArr[i][j],
						{z : 0},
						1.5, 
						Exponential.easeOut,
						i * ease + Math.random() * 0.3);
					
					ease -= 0.0003;
				}
			}
			
			// Radiation Blur Effect setting
			radiationBlurPlane.x = - imgSprite.width / 2;
			radiationBlurPlane.alpha = 0;
			
			Tweensy.to(
				radiationBlurPlane,
				{x : imgSprite.width / 2},
				3.75,
				//Quadratic.easeInOut,
				Sine.easeIn,
				1.25
				);
			
			Tweensy.to(
				radiationBlurPlane,
				{alpha : 1},
				1.5,
				Linear.easeNone,
				0.75);
				
			Tweensy.to(
				radiationBlurPlane,
				{alpha : 0},
				1.5, 
				Linear.easeNone,
				4.5 );
				
			// line setting
			linePlane.x = - imgSprite.width / 2;
			Tweensy.to(
				linePlane,
				{x : imgSprite.width / 2},
				3.75,
				//Quadratic.easeInOut,
				Sine.easeIn,
				1.25
				);
				
			Tweensy.to(
				linePlane,
				{alpha : 1},
				1.5, 
				Linear.easeNone,
				0.75);
				
			Tweensy.to(
				linePlane,
				{alpha : 0},
				1.5, 
				Linear.easeNone,
				4.5 );
				
			// setting camera
			camera.x = -400;
			camera.y = -400;
			camera.z = -1000;
			
			Tweensy.to(
				camera,
				{z : -300},
				6, 
				null);
						
			Tweensy.to(
				camera,
				{x : 0, y : 0},
				5.5, 
				Exponential.easeInOut,
				1.25,
				null,
				function():void
				{
					// stop PV3D Rendering
					stopRendering();
					isMotion = false;
				});
		}
		
	}
}
